/********************
 * 2004 Jalview Reengineered
 * Barton Group
 * Dundee University
 *
 * AM Waterhouse
 *******************/




package jalview.gui;

import jalview.jbgui.GAlignFrame;
import jalview.schemes.*;
import jalview.datamodel.*;
import jalview.analysis.*;
import jalview.io.*;
import jalview.ws.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.print.*;
import javax.swing.*;
import javax.swing.event.*;
import java.util.*;
import  java.awt.datatransfer.*;


public class AlignFrame extends GAlignFrame
{
  final AlignmentPanel alignPanel;
  final AlignViewport viewport;
  public AlignFrame(AlignmentI al)
  {
    super();
    viewport = new AlignViewport(al,true,true,false);

    String fontName = jalview.bin.Cache.getProperty("FONT_NAME");
    String fontStyle= jalview.bin.Cache.getProperty("FONT_STYLE");
    String fontSize = jalview.bin.Cache.getProperty("FONT_SIZE");
    if(fontName!=null && fontStyle!=null && fontSize!=null)
      viewport.setFont( new Font(fontName,Integer.parseInt(fontStyle),Integer.parseInt(fontSize)) );


    alignPanel = new AlignmentPanel(this, viewport);
    getContentPane().add(alignPanel, java.awt.BorderLayout.CENTER);

    addInternalFrameListener(new InternalFrameAdapter()
   {
     public void internalFrameActivated(InternalFrameEvent evt)
     {
          javax.swing.SwingUtilities.invokeLater(new Runnable()
          {
            public void run()
            {      alignPanel.requestFocus();    }
          });

     }
   });

  }

  protected void saveAs_actionPerformed(ActionEvent e)
  {
    String suffix = "";
    if(e.getActionCommand().equals("FASTA"))
      suffix = "fa";
    else if(e.getActionCommand().equals("MSF"))
      suffix = "msf";
    else if(e.getActionCommand().equals("CLUSTAL"))
      suffix = "aln";
    else if(e.getActionCommand().equals("BLC"))
      suffix = "blc";
    else if(e.getActionCommand().equals("PIR"))
      suffix = "pir";
    else if(e.getActionCommand().equals("PFAM"))
      suffix = "pfam";

    JalviewFileChooser chooser = new JalviewFileChooser(jalview.bin.Cache.getProperty("LAST_DIRECTORY")
        , new String[]{suffix}, e.getActionCommand()+" file");
    chooser.setFileView(new JalviewFileView());
    chooser.setDialogTitle("Save Alignment to file - "+e.getActionCommand() +" format.");
    chooser.setToolTipText("Save");
    int value = chooser.showSaveDialog(this);
    if(value == JalviewFileChooser.APPROVE_OPTION)
    {
      String choice =  chooser.getSelectedFile().getPath();
      jalview.bin.Cache.setProperty("LAST_DIRECTORY", choice);
      String output = FormatAdapter.get(e.getActionCommand(), viewport.getAlignment().getSequences());
      try{
        java.io.PrintWriter out = new java.io.PrintWriter( new java.io.FileWriter( choice )  );
        out.println(output);
        out.close();
      }
      catch(Exception ex){}
    }

  }

  protected void outputText_actionPerformed(ActionEvent e)
  {
     CutAndPasteTransfer cap = new CutAndPasteTransfer(false);
     JInternalFrame frame = new JInternalFrame();
     cap.formatForOutput();
     frame.setContentPane(cap);
     Desktop.addInternalFrame(frame, "Alignment output - "+e.getActionCommand(), 600, 500);
     cap.setText( FormatAdapter.get(e.getActionCommand(), viewport.getAlignment().getSequences()));
  }

  protected void htmlMenuItem_actionPerformed(ActionEvent e)
  {
    HTMLOutput htmlOutput = new HTMLOutput(viewport);
    htmlOutput = null;
  }

  protected void createPNG_actionPerformed(ActionEvent e)
  {
    alignPanel.makePNG();
  }

  protected void epsFile_actionPerformed(ActionEvent e)
  {
    alignPanel.makeEPS();
  }


  public void printMenuItem_actionPerformed(ActionEvent e)
  {
    //Putting in a thread avoids Swing painting problems
    PrintThread thread = new PrintThread();
    thread.start();
  }

  class PrintThread extends Thread
  {
    public void run()
    {
      PrinterJob printJob = PrinterJob.getPrinterJob();
      PageFormat pf = printJob.pageDialog(printJob.defaultPage());
      printJob.setPrintable(alignPanel, pf);
      if (printJob.printDialog())
      {
        try
        {
          printJob.print();
        }
        catch (Exception PrintException)
        {
          PrintException.printStackTrace();
        }
      }
    }

  }




  public void closeMenuItem_actionPerformed(ActionEvent e)
  {
    try{
      this.setClosed(true);
    }catch(Exception ex){}
  }

  Stack historyList = new Stack();
  Stack redoList = new Stack();
  JMenuBar jMenuBar1 = new JMenuBar();

  void updateEditMenuBar()
  {
    if(historyList.size()>0)
     {
       undoMenuItem.setEnabled(true);
       Object [] history = (Object[])historyList.get(0);
       undoMenuItem.setText("Undo "+history[0]);
     }
    else
    {
      undoMenuItem.setEnabled(false);
      undoMenuItem.setText("Undo");
    }

    if(redoList.size()>0)
     {
       redoMenuItem.setEnabled(true);
       Object [] history = (Object[])redoList.get(0);
       redoMenuItem.setText("Redo "+history[0]);
     }
    else
    {
      redoMenuItem.setEnabled(false);
      redoMenuItem.setText("Redo");
    }
  }

  public void addHistoryItem(String type)
  {
    // must make sure we add new sequence objects her, not refs to the existing sequences
    redoList.clear();
    SequenceI[] seq = new SequenceI[viewport.getAlignment().getHeight()];
    for(int i=0; i<viewport.getAlignment().getHeight(); i++)
    {
      seq[i] = new Sequence( viewport.getAlignment().getSequenceAt(i).getName(),
                             viewport.getAlignment().getSequenceAt(i).getSequence());
    }

    historyList.add(0, new Object[]{type,  seq} );
    updateEditMenuBar();
  }

  protected void undoMenuItem_actionPerformed(ActionEvent e)
  {
    Object [] history = (Object[])historyList.remove(0);
    // add the redo state before continuing!!
    SequenceI[] seq = new SequenceI[viewport.getAlignment().getHeight()];
    for (int i = 0; i < viewport.getAlignment().getHeight(); i++)
    {
      seq[i] = new Sequence(viewport.getAlignment().getSequenceAt(i).getName(),
                            viewport.getAlignment().getSequenceAt(i).
                            getSequence());
    }
    /////////

    redoList.add(0, new Object[] {history[0], seq});

      seq = (SequenceI[]) history[1];
      viewport.setAlignment( new Alignment(seq) );
      updateEditMenuBar();
      alignPanel.RefreshPanels();
  }

  public void moveSelectedSequences(boolean up)
  {
    SequenceGroup sg = viewport.getSelectionGroup();
    if (sg == null)
      return;

    if (up)
    {
      for (int i = 1; i < viewport.alignment.getHeight(); i++)
      {
        SequenceI seq = viewport.alignment.getSequenceAt(i);
        if (!sg.sequences.contains(seq))
          continue;

        SequenceI temp = viewport.alignment.getSequenceAt(i - 1);
        if (sg.sequences.contains(temp))
          continue;

        viewport.alignment.getSequences().setElementAt(temp, i);
        viewport.alignment.getSequences().setElementAt(seq, i - 1);
      }
    }
    else
    {
      for (int i = viewport.alignment.getHeight() - 2; i > -1; i--)
      {
        SequenceI seq = viewport.alignment.getSequenceAt(i);
        if (!sg.sequences.contains(seq))
          continue;

        SequenceI temp = viewport.alignment.getSequenceAt(i + 1);
        if (sg.sequences.contains(temp))
          continue;

        viewport.alignment.getSequences().setElementAt(temp, i);
        viewport.alignment.getSequences().setElementAt(seq, i + 1);
      }
    }

    alignPanel.RefreshPanels();
  }



  protected void copy_actionPerformed(ActionEvent e)
  {
   if(viewport.getSelectionGroup()==null)
     return;

   SequenceGroup sg = viewport.getSelectionGroup();

     Clipboard c = Toolkit.getDefaultToolkit().getSystemClipboard();
     StringBuffer buffer= new StringBuffer();

       for(int i=0; i<sg.getSize(); i++)
       {
         SequenceI seq = sg.getSequenceAt(i);
         buffer.append( seq.getName()+"\t"+seq.findPosition( sg.getStartRes() ) +"\t"
                        +seq.findPosition( sg.getEndRes() )+ "\t"
                        +sg.getSequenceAt(i).getSequence(sg.getStartRes(), sg.getEndRes()+1)+"\n");
       }
     c.setContents( new StringSelection( buffer.toString()) , null ) ;

  }


  protected void pasteNew_actionPerformed(ActionEvent e)
  {
    paste(true);
  }

  protected void pasteThis_actionPerformed(ActionEvent e)
  {
    addHistoryItem("Paste");
    paste(false);
  }

  void paste(boolean newAlignment)
  {
    try{
      Clipboard c = Toolkit.getDefaultToolkit().getSystemClipboard();
      Transferable contents = c.getContents(this);
      if (contents == null)
        return;

      String str = (String) contents.getTransferData(DataFlavor.stringFlavor);
      StringTokenizer st = new StringTokenizer(str);
      ArrayList seqs = new ArrayList();
      while (st.hasMoreElements())
      {
        String name = st.nextToken();
        int start = Integer.parseInt(st.nextToken());
        int end = Integer.parseInt(st.nextToken());
        Sequence sequence = new Sequence(name,st.nextToken(), start, end);

        if(!newAlignment)
          viewport.alignment.addSequence(sequence);
        else
          seqs.add(sequence);
      }

      if(newAlignment)
      {
        SequenceI[] newSeqs = new SequenceI[seqs.size()];
        seqs.toArray(newSeqs);
        AlignFrame af = new AlignFrame(new Alignment(newSeqs));
        int newHeight = newSeqs.length * af.viewport.getCharHeight() + 200;
        if (newHeight > 500)
          newHeight = 500;
        Desktop.addInternalFrame(af, "Copied sequences", 700, newHeight);
      }
      else
      {
        viewport.setEndSeq(viewport.alignment.getHeight());
        viewport.alignment.getWidth();
        alignPanel.RefreshPanels();
      }

    }catch(Exception ex){}// could be anything being pasted in here

  }


  protected void cut_actionPerformed(ActionEvent e)
  {
    copy_actionPerformed(null);
    delete_actionPerformed(null);
  }

  protected void delete_actionPerformed(ActionEvent e)
  {
    addHistoryItem("Delete");
    if (viewport.getSelectionGroup() == null)
      return;

     SequenceGroup sg = viewport.getSelectionGroup();
     for (int i=0;i < sg.sequences.size(); i++)
     {
       SequenceI seq = sg.getSequenceAt(i);
       int index = viewport.getAlignment().findIndex(seq);
       seq.deleteChars(sg.getStartRes(), sg.getEndRes()+1);

       if(seq.getSequence().length()<1)
          viewport.getAlignment().deleteSequence(seq);
      else
          viewport.getAlignment().getSequences().setElementAt(seq, index);
     }

     viewport.setSelectionGroup(null);
     viewport.alignment.deleteGroup(sg);
     viewport.resetSeqLimits( alignPanel.seqPanel.seqCanvas.getHeight());
     if(viewport.getAlignment().getHeight()<1)
     try
     {
       this.setClosed(true);
     }catch(Exception ex){}
     alignPanel.RefreshPanels();

  }



  protected void redoMenuItem_actionPerformed(ActionEvent e)
  {
     Object [] history = (Object[])redoList.remove(0);
     SequenceI[] seq = (SequenceI[]) history[1];
     viewport.setAlignment( new Alignment(seq) );
     updateEditMenuBar();
     alignPanel.RefreshPanels();
  }


  public void groupsMenuItem_actionPerformed(ActionEvent e)
  {
    GroupEditor geditor = new GroupEditor(viewport, alignPanel);
    JInternalFrame frame = new JInternalFrame();
    frame.setContentPane(geditor);
    Desktop.addInternalFrame(frame, "Group editor", 710, 410);
    frame.setResizable(false);
  }

  protected void deleteGroups_actionPerformed(ActionEvent e)
  {
    viewport.alignment.deleteAllGroups();
    viewport.setSelectionGroup(null);

    alignPanel.RefreshPanels();
  }



  public void selectAllSequenceMenuItem_actionPerformed(ActionEvent e)
  {
    SequenceGroup sg = new SequenceGroup();
    for (int i=0; i<viewport.getAlignment().getSequences().size(); i++)
      sg.addSequence( viewport.getAlignment().getSequenceAt(i));
    sg.setEndRes(viewport.alignment.getWidth());
    viewport.setSelectionGroup(sg);
    PaintRefresher.Refresh(null);
  }

  public void deselectAllSequenceMenuItem_actionPerformed(ActionEvent e)
  {
    viewport.setSelectionGroup(null);
    PaintRefresher.Refresh(null);
  }

  public void invertSequenceMenuItem_actionPerformed(ActionEvent e)
  {
    SequenceGroup sg = viewport.getSelectionGroup();
    for (int i=0; i<viewport.getAlignment().getSequences().size(); i++)
      sg.addOrRemove (viewport.getAlignment().getSequenceAt(i));

    PaintRefresher.Refresh(null);
  }


  public void deselectAllColumnsMenuItem_actionPerformed(ActionEvent e)
  {
    viewport.getColumnSelection().clear();
    repaint();
  }

  public void remove2LeftMenuItem_actionPerformed(ActionEvent e)
  {
    addHistoryItem("delete columns");
    ColumnSelection colSel = viewport.getColumnSelection();
    if (colSel.size() > 0)
    {
      int min = colSel.getMin();
      viewport.getAlignment().trimLeft(min);
      colSel.compensateForEdit(0,min);
      alignPanel.RefreshPanels();
    }
  }

  public void remove2RightMenuItem_actionPerformed(ActionEvent e)
  {
    addHistoryItem("delete columns");
    ColumnSelection colSel = viewport.getColumnSelection();
    if (colSel.size() > 0)
    {
      int max = colSel.getMax();
      if(max>1)
        viewport.getAlignment().trimRight(max);

      alignPanel.RefreshPanels();
    }

  }

  public void removeGappedColumnMenuItem_actionPerformed(ActionEvent e)
  {
    addHistoryItem("delete gapped columns");
    viewport.getAlignment().removeGaps();
    alignPanel.RefreshPanels();
  }

  public void removeAllGapsMenuItem_actionPerformed(ActionEvent e)
  {
    addHistoryItem("delete all gaps");
    SequenceI current;
    int jSize;
    for (int i=0; i < viewport.getAlignment().getSequences().size();i++)
    {
      current = viewport.getAlignment().getSequenceAt(i);
      jSize = current.getLength();
      for (int j=0; j < jSize; j++)
        if(jalview.util.Comparison.isGap(current.getCharAt(j)))
        {
          current.deleteCharAt(j);
          j--;
          jSize--;
        }
    }

    alignPanel.RefreshPanels();
  }

  public void setGapCharMenuItem_actionPerformed(ActionEvent e)
  {
    char thisChar = '-';
    char nextChar = '.';
    if(viewport.getGapCharacter()=='-')
    {
      thisChar = '.';
      nextChar = '-';
    }
    setGapCharMenuItem.setText("Set gap character to \""+nextChar+"\"");
    viewport.setGapCharacter(thisChar);
    alignPanel.RefreshPanels();
  }

  public void findMenuItem_actionPerformed(ActionEvent e)
  {
      JInternalFrame frame = new JInternalFrame();
      Finder finder = new Finder(viewport, alignPanel, frame);
      frame.setContentPane(finder);
     Desktop.addInternalFrame(frame, "Find", 340,110);

  }


  public void font_actionPerformed(ActionEvent e)
  {
   // JOptionPane op = new JOptionPane();
    JInternalFrame frame = new JInternalFrame();
    FontChooser fc = new FontChooser( alignPanel, frame );
    frame.setContentPane(fc);
    Desktop.addInternalFrame(frame, "Change Font", 480,100);
  }

  protected void fullSeqId_actionPerformed(ActionEvent e)
  {
    viewport.setShowFullId( fullSeqId.isSelected() );

    alignPanel.idPanel.idCanvas.setPreferredSize( alignPanel.calculateIdWidth() );
    alignPanel.RefreshPanels();
  }

  protected void colourTextMenuItem_actionPerformed(ActionEvent e)
  {
      viewport.setColourText( colourTextMenuItem.isSelected() );
      alignPanel.RefreshPanels();
  }

  protected void wrapMenuItem_actionPerformed(ActionEvent e)
  {
    viewport.setWrapAlignment( wrapMenuItem.isSelected() );
    alignPanel.setWrapAlignment( wrapMenuItem.isSelected() );
  }


  public void viewBoxesMenuItem_actionPerformed(ActionEvent e)
  {
    viewport.setShowBoxes( viewBoxesMenuItem.isSelected() );
    alignPanel.RefreshPanels();
  }

  public void viewTextMenuItem_actionPerformed(ActionEvent e)
  {
    viewport.setShowText( viewTextMenuItem.isSelected() );
    alignPanel.RefreshPanels();
  }


  protected void renderGapsMenuItem_actionPerformed(ActionEvent e)
  {
    viewport.setRenderGaps(renderGapsMenuItem.isSelected());
    alignPanel.RefreshPanels();
  }

  public void sequenceFeatures_actionPerformed(ActionEvent evt)
  {
    viewport.showSequenceFeatures(sequenceFeatures.isSelected());
    if(viewport.showSequenceFeatures && !((Alignment)viewport.alignment).featuresAdded)
    {
         AlignmentUtil.fetchSequenceFeatures( viewport.alignment , alignPanel);
         ((Alignment)viewport.alignment).featuresAdded = true;
    }
    alignPanel.RefreshPanels();
  }

  public void consensusGraphMenuItem_actionPerformed(ActionEvent e)
  {
    alignPanel.setGraphPanelVisible( consensusGraphMenuItem.isSelected() );
  }

  public void overviewMenuItem_actionPerformed(ActionEvent e)
  {
    if (alignPanel.overviewPanel != null)
      return;

    JInternalFrame frame = new JInternalFrame();
    OverviewPanel overview = alignPanel.getOverviewPanel();
   try{
     overview = new OverviewPanel(alignPanel, viewport);
     frame.setContentPane(overview);
    Desktop.addInternalFrame(frame, "Overview " + this.getTitle(),
                             frame.getWidth(), frame.getHeight());
    frame.pack();
    frame.addInternalFrameListener(new javax.swing.event.InternalFrameAdapter()
    { public void internalFrameClosed(javax.swing.event.InternalFrameEvent evt)
      {
            alignPanel.setOverviewPanel(null);
      };
    });
    alignPanel.setOverviewPanel( overview );

  }catch(java.lang.OutOfMemoryError ex)
   {
     JOptionPane.showInternalMessageDialog(this, "Sequence alignment too large to\nproduce overview image!!"
                                   +"\nTry reducing the font size.",
                                   "Out of memory", JOptionPane.WARNING_MESSAGE);
   }


  }

  protected void noColourmenuItem_actionPerformed(ActionEvent e)
  {
    changeColour( null );
  }


  public void clustalColour_actionPerformed(ActionEvent e)
  {
    abovePIDThreshold.setSelected(false);
    changeColour(new ClustalxColourScheme(viewport.alignment.getSequences(), viewport.alignment.getWidth()));
  }

  public void zappoColour_actionPerformed(ActionEvent e)
  {
    changeColour(new ZappoColourScheme());
  }

  public void taylorColour_actionPerformed(ActionEvent e)
  {
    changeColour(new TaylorColourScheme());
  }


  public void hydrophobicityColour_actionPerformed(ActionEvent e)
  {
    changeColour( new HydrophobicColourScheme() );
  }

  public void helixColour_actionPerformed(ActionEvent e)
  {
    changeColour(new HelixColourScheme() );
  }


  public void strandColour_actionPerformed(ActionEvent e)
  {
    changeColour(new StrandColourScheme());
  }


  public void turnColour_actionPerformed(ActionEvent e)
  {
    changeColour(new TurnColourScheme());
  }


  public void buriedColour_actionPerformed(ActionEvent e)
  {
    changeColour(new BuriedColourScheme() );
  }

  public void nucleotideColour_actionPerformed(ActionEvent e)
  {
    changeColour(new NucleotideColourScheme());
  }


  protected void applyToAllGroups_actionPerformed(ActionEvent e)
  {
    viewport.setColourAppliesToAllGroups(applyToAllGroups.isSelected());
  }



  void changeColour(ColourSchemeI cs)
  {

    if(viewport.getColourAppliesToAllGroups())
    {
      Vector groups = viewport.alignment.getGroups();
      for(int i=0; i<groups.size(); i++)
      {
        SequenceGroup sg = (SequenceGroup)groups.elementAt(i);
        sg.cs = cs;

        if(abovePIDThreshold.isSelected())
          abovePIDThreshold_actionPerformed(null);
        else if( viewport.getConservationSelected() )
        {
          Conservation c = new Conservation("Group",
                                            ResidueProperties.propHash, 3,
                                            sg.sequences, sg.getStartRes(),
                                            sg.getEndRes());
          c.calculate();
          c.verdict(false, 100);
          ConservationColourScheme ccs = new ConservationColourScheme(c, sg.cs);

          sg.cs = ccs;
        }

      }
    }


    if ( viewport.getAbovePIDThreshold())
    {
      int threshold = 0;
      threshold = Desktop.setPIDSliderSource(alignPanel, cs, "Background");
      Desktop.hideConservationSlider();

      if (cs instanceof ResidueColourScheme)
        ( (ResidueColourScheme) cs).setThreshold(threshold);
      else if (cs instanceof ScoreColourScheme)
        ( (ScoreColourScheme) cs).setThreshold(threshold);

      viewport.setGlobalColourScheme(cs);

    }
    else
    if (cs instanceof ResidueColourScheme)
      ( (ResidueColourScheme) cs).setThreshold(0);
    else if (cs instanceof ScoreColourScheme)
      ( (ScoreColourScheme) cs).setThreshold(0);




if ( viewport.getConservationSelected() )
 {
   ConservationColourScheme ccs = null;

   Alignment al = (Alignment) viewport.alignment;
   Conservation c = new Conservation("All",
                                     ResidueProperties.propHash, 3,
                                     al.getSequences(), 0,
                                     al.getWidth());

   c.calculate();
   c.verdict(false, 100);

   ccs = new ConservationColourScheme(c, cs);

   int threshold = Desktop.setConservationSliderSource(alignPanel, ccs,
       "Background");

   ccs.inc = threshold;

   viewport.setGlobalColourScheme(ccs);

  }
  else
       viewport.setGlobalColourScheme( cs );

    alignPanel.RefreshPanels();
  }


  protected  void conservationMenuItem_actionPerformed(ActionEvent e)
  {
    viewport.setConservationSelected(conservationMenuItem.isSelected());
    if(coloursChanging)
      return;

    coloursChanging = true;

    viewport.setAbovePIDThreshold(false);
    abovePIDThreshold.setSelected(false);
    Desktop.hidePIDSlider();
    if(!viewport.getConservationSelected())
      Desktop.hideConservationSlider();

   ColourSchemeI cs = viewport.getGlobalColourScheme();
   if(cs instanceof ConservationColourScheme )
     changeColour( ((ConservationColourScheme)cs).cs );
    else
      changeColour( cs );

    coloursChanging = false;
  }

  boolean coloursChanging = false;
  public void abovePIDThreshold_actionPerformed(ActionEvent e)
  {
    if(coloursChanging)
      return;

    coloursChanging = true;
    viewport.setAbovePIDThreshold(abovePIDThreshold.isSelected());

    conservationMenuItem.setSelected(false);
    viewport.setConservationSelected(false);
    Desktop.hideConservationSlider();

    if(!viewport.getAbovePIDThreshold())
      Desktop.hidePIDSlider();


    ColourSchemeI cs = viewport.getGlobalColourScheme();
    if(cs instanceof ConservationColourScheme )
        changeColour( ((ConservationColourScheme)cs).cs );
    else
        changeColour( cs );

    coloursChanging = false;
  }



  public void userDefinedColour_actionPerformed(ActionEvent e)
  {
    JInternalFrame frame = new JInternalFrame();
    UserDefinedColours chooser = new UserDefinedColours( frame, alignPanel, null);
    frame.setContentPane(chooser);
    Desktop.addInternalFrame(frame,"User defined colours", 450,540 );
    frame.setResizable(false);
    frame.setIconifiable(false);
    frame.setMaximizable(false);
  }

  public void PIDColour_actionPerformed(ActionEvent e)
  {
    changeColour( new PIDColourScheme() );
  }


  public void BLOSUM62Colour_actionPerformed(ActionEvent e)
  {
    changeColour(new Blosum62ColourScheme(viewport) );
  }



  public void sortPairwiseMenuItem_actionPerformed(ActionEvent e)
  {
    addHistoryItem("sort");
    AlignmentSorter.sortByPID(viewport.getAlignment(), viewport.getAlignment().getSequenceAt(0));
    alignPanel.RefreshPanels();
  }

  public void sortIDMenuItem_actionPerformed(ActionEvent e)
  {
    addHistoryItem("sort");
    AlignmentSorter.sortByID( viewport.getAlignment() );
    alignPanel.RefreshPanels();
  }

  public void sortGroupMenuItem_actionPerformed(ActionEvent e)
  {
    addHistoryItem("sort");
    AlignmentSorter.sortByGroup(viewport.getAlignment());
    AlignmentSorter.sortGroups(viewport.getAlignment());
    alignPanel.RefreshPanels();
  }

  public void sortTreeOrderMenuItem_actionPerformed(ActionEvent e)
  {
    addHistoryItem("sort");
    if(viewport.getCurrentTree()==null)
      return;

    AlignmentSorter.sortByTree(viewport.getAlignment(), viewport.getCurrentTree());
    alignPanel.RefreshPanels();
  }

  public void removeRedundancyMenuItem_actionPerformed(ActionEvent e)
  {
    RedundancyPanel sp = new RedundancyPanel(alignPanel);
    JInternalFrame frame = new JInternalFrame();
    frame.setContentPane(sp);
    Desktop.addInternalFrame(frame, "Redundancy threshold selection", 400, 100);
    frame.setMaximizable(false);
    frame.setResizable(false);

  }

  public void pairwiseAlignmentMenuItem_actionPerformed(ActionEvent e)
  {
    if(viewport.getSelectionGroup().getSize()<2)
      JOptionPane.showInternalMessageDialog(this, "You must select at least 2 sequences.", "Invalid Selection", JOptionPane.WARNING_MESSAGE);
    else
    {
      JInternalFrame frame = new JInternalFrame();
      frame.setContentPane(new PairwiseAlignPanel(viewport));
      Desktop.addInternalFrame(frame, "Pairwise Alignment", 600, 500);
    }
  }

  public void PCAMenuItem_actionPerformed(ActionEvent e)
  {

    if( (viewport.getSelectionGroup()!=null && viewport.getSelectionGroup().getSize()<4 && viewport.getSelectionGroup().getSize()>0)
       || viewport.getAlignment().getHeight()<4)
    {
      JOptionPane.showInternalMessageDialog(this, "Principal component analysis must take\n"
                                    +"at least 4 input sequences.",
                                    "Sequence selection insufficient",
                                    JOptionPane.WARNING_MESSAGE);
      return;
    }

    try{
      PCAPanel pcaPanel = new PCAPanel(viewport, null);
      JInternalFrame frame = new JInternalFrame();
      frame.setContentPane(pcaPanel);
      Desktop.addInternalFrame(frame, "Principal component analysis", 400, 400);
   }catch(java.lang.OutOfMemoryError ex)
   {
     JOptionPane.showInternalMessageDialog(this, "Too many sequences selected\nfor Principal Component Analysis!!",
                                   "Out of memory", JOptionPane.WARNING_MESSAGE);
   }


  }

  public void averageDistanceTreeMenuItem_actionPerformed(ActionEvent e)
  {
    NewTreePanel("AV", "PID", "Average distance tree using PID");
  }

  public void neighbourTreeMenuItem_actionPerformed(ActionEvent e)
  {
    NewTreePanel("NJ", "PID", "Neighbour joining tree using PID");
  }


  protected void njTreeBlosumMenuItem_actionPerformed(ActionEvent e)
  {
    NewTreePanel("NJ", "BL", "Neighbour joining tree using BLOSUM62");
  }

  protected void avTreeBlosumMenuItem_actionPerformed(ActionEvent e)
  {
    NewTreePanel("AV", "BL", "Average distance tree using BLOSUM62PID");
  }

  void NewTreePanel(String type, String pwType, String title)
  {
    //are the sequences aligned?
    if(!viewport.alignment.isAligned())
    {
      JOptionPane.showMessageDialog(Desktop.desktop, "The sequences must be aligned before creating a tree.",
                                    "Sequences not aligned", JOptionPane.WARNING_MESSAGE);
      return;
    }

    TreePanel tp=null;
   if (viewport.getSelectionGroup() != null && viewport.getSelectionGroup().getSize() > 3)
   {
     tp = new TreePanel(viewport, viewport.getSelectionGroup().sequences,type, pwType,
                         0, viewport.alignment.getWidth());
   }
   else
   {
     tp = new TreePanel(viewport, viewport.getAlignment().getSequences(),
                         type, pwType,  0, viewport.alignment.getWidth());
   }

   Desktop.addInternalFrame(tp, title, 600, 500);
  }




  public void clustalAlignMenuItem_actionPerformed(ActionEvent e)
  {
    JInternalFrame frame = new JInternalFrame();
    ClustalThread ct = new ClustalThread(frame);
    Thread t = new Thread(ct);
    t.start();
    frame.setContentPane(ct);
     Desktop.addInternalFrame(frame, title, 300, 80);

  }

  class ClustalThread extends JPanel implements Runnable
  {
    Image [] image;
    int imageIndex = 0;
    boolean webServiceRunning = false;
    JInternalFrame frame;
    public ClustalThread(JInternalFrame frame)
    {
      this.frame = frame;
      image = new Image[9];
      for(int i=0; i<9; i++)
      {
        java.net.URL url = getClass().getResource("/images/dna" + (i+1) + ".gif");
        if (url != null)
        {
          image[i] = java.awt.Toolkit.getDefaultToolkit().createImage(url);
          MediaTracker mt = new MediaTracker(this);
          mt.addImage(image[i], i);
          try{mt.waitForID(i);}
          catch(Exception ex){}
        }
      }
      DNATwirler twirler = new DNATwirler();
      twirler.start();
      webServiceRunning = true;
    }

    class DNATwirler extends Thread
    {
      public void run()
      {
        while(webServiceRunning)
        {
          try{
            Thread.sleep(100);
            imageIndex++;
            imageIndex %=9;
            repaint();
          }
          catch(Exception ex){}
        }
      }
    }

    // JBPNote
    // Should check to see if the server settings are valid
    // Need visual-delay indication here.
    public void run()
       {
         jalview.ws.Jemboss jemboss = new jalview.ws.Jemboss();
         Vector sv = viewport.getAlignment().getSequences();
         SequenceI[] seqs = new SequenceI[sv.size()];

         int i = 0;
         do
         {
           seqs[i] = (SequenceI) sv.elementAt(i);
         }
         while (++i < sv.size());

         SequenceI[] alignment = jemboss.clustalW(seqs); // gaps removed within method

         if (alignment != null)
         {
           AlignFrame af = new AlignFrame(new Alignment(alignment));
          af.clustalColour.setSelected(true);
          af.clustalColour_actionPerformed(null);
           Desktop.addInternalFrame(af, getTitle().concat(" - ClustalW Alignment"),
                                    700, 500); // JBPNote - is there a new window geom. property ?
         }
         else
           JOptionPane.showMessageDialog(Desktop.desktop, "Problem obtaining clustal alignment", "Web service error",
                                         JOptionPane.WARNING_MESSAGE);

         webServiceRunning = false;
         try{
           frame.setClosed(true);
         }catch(Exception ex){}
       }

       public void paintComponent(Graphics g)
       {
         g.setColor(Color.white);
         g.fillRect(0,0,getWidth(), getHeight());
         if(image!=null)
         {
           g.drawImage(image[imageIndex],10,10,this);
         }
         g.setFont(new Font("Arial", Font.BOLD, 12));
         g.setColor(Color.black);
         g.drawString("Clustal Alignment Web Service running", 30,30);
       }
  }
  public void jpredMenuItem_actionPerformed(ActionEvent e)
    {
    //  JInternalFrame frame = new JInternalFrame();
//      JPredClient ct = new JPredClient((SequenceI)
    ///                                   viewport.getAlignment().getSequences().elementAt(0));
    }

    protected void LoadtreeMenuItem_actionPerformed(ActionEvent e) {
    // Pick the tree file
    JalviewFileChooser chooser = new JalviewFileChooser(jalview.bin.Cache.
        getProperty("LAST_DIRECTORY"));
    chooser.setFileView(new JalviewFileView());
    chooser.setDialogTitle("Select a newick-like tree file");
    chooser.setToolTipText("Load a tree file");
    int value = chooser.showOpenDialog(null);
    if (value == JalviewFileChooser.APPROVE_OPTION)
    {
      String choice = chooser.getSelectedFile().getPath();
      jalview.bin.Cache.setProperty("LAST_DIRECTORY", choice);
      TreePanel treepanel = null;
      try
      {
        jalview.io.NewickFile fin = new jalview.io.NewickFile(choice, "File");
        fin.parse();

        if (fin.getTree() != null)
        {
          TreePanel tp = null;
          tp = new TreePanel(viewport, viewport.getAlignment().getSequences(),
                             fin, "FromFile", choice);
          Desktop.addInternalFrame(tp, title, 600, 500);
        }
      }
      catch (Exception ex)
      {
        JOptionPane.showMessageDialog(Desktop.desktop,
                                      "Problem reading tree file",
                                      ex.getMessage(),
                                      JOptionPane.WARNING_MESSAGE);
        ex.printStackTrace();
      }
    }
  }

}
